# _*_ coding: utf-8 _*_
#
# hspylib v0.11.61
#
# Package: main.hspylib.modules.cli.tui.extra.minput
"""Package initialization."""

__all__ = [
  'access_type', 
  'field_builder', 
  'form_builder', 
  'form_field', 
  'input_type', 
  'input_validator', 
  'minput', 
  'minput_utils'
]
__version__ = '0.11.61'
